#version 330

layout (location = 0) in vec3 inPosition;
layout (location = 1) in vec2 inCoord;
layout (location = 2) in vec3 inNormal;
layout (location = 3) in vec3 inTangent;

smooth out vec2 vTexCoord;
smooth out vec3 vNormal;
smooth out vec3 vWorldPos;
smooth out vec3 vTangent;
smooth out vec4 shadowCoordinates;
smooth out vec4 projectedTextureCoordinates;

uniform mat4 projectionMatrix;
uniform mat4 modelMatrix;
uniform mat4 viewMatrix; 
uniform mat4 normalMatrix;
uniform mat4 depthBiasMVP;
uniform mat4 projectedTextureBiasMVP;

uniform int normalMappingOn; 

void main()
{
	mat4 modelViewProjectionMatrix = projectionMatrix * viewMatrix * modelMatrix;
	gl_Position = modelViewProjectionMatrix * vec4(inPosition, 1.0);

	vTexCoord = inCoord;
	vNormal = normalize((normalMatrix*vec4(inNormal, 0.0)).xyz);
	vWorldPos = (modelMatrix*vec4(inPosition, 1.0)).xyz;
	
	if (normalMappingOn == 1) {
	    vTangent = (modelMatrix * vec4(inTangent, 0.0)).xyz;
	} else {
		vTangent = vec3(0.0, 0.0, 0.0);
	}
	
	shadowCoordinates = depthBiasMVP * vec4(vWorldPos, 1.0);
	projectedTextureCoordinates = projectedTextureBiasMVP * vec4(vWorldPos, 1.0);
}